/*
 * Copyright (c) 2014 UPLEX - Nils Goroll Systemoptimierung
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS _AS IS_ AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "dcs_config.h"

#include <errno.h>

#include "dcs_classifier.h"
#include "dcs_type.h"
	    
#define check_type_id(type_id, ret)					\
	do {								\
		if ((type_id < 0) || (type_id > (DCS_TYPE_COUNT - 1))) { \
			errno = EINVAL;					\
			return (ret);					\
		}							\
	} while(0)

const char *
dcs_type_name(int type_id /* enum dcs_type */) {
	check_type_id(type_id, 0);
	return dcs_type[type_id];
}

/*
 * useful meta-classification to just three string constants:
 *
 */
enum dcs_type_class {
	T_CLASS_MISSING = 0,
	T_CLASS_MOB,
	T_CLASS_TAB,
	T_CLASS_DSK,
	_T_CLASS_LIMIT
};

/*
 * the class string "smartphone" might not be optimal,
 * - read this as "mobile device".
 */
const char * const dcs_type_class_str[_T_CLASS_LIMIT] = {
	[T_CLASS_MISSING]	= "desktop",
	[T_CLASS_MOB]		= "smartphone",
	[T_CLASS_TAB]		= "tablet",
	[T_CLASS_DSK]		= "desktop"
};
	
enum dcs_type dcs_type2class[DCS_TYPE_COUNT] = {
	[NB_T_UNIDENTIFIED]		= T_CLASS_DSK,
	[NB_T_BOT]			= T_CLASS_DSK,
	[NB_T_CAMERA]			= T_CLASS_MOB,
	[NB_T_CE_DEVICE]		= T_CLASS_MOB,	// android 1.5; en-us; nimble
	[NB_T_COMPUTER]			= T_CLASS_DSK,	// macintosh*intel*mac os
	[NB_T_DESKTOP_BROWSER]		= T_CLASS_DSK,
	[NB_T_DEVTOOL]			= T_CLASS_DSK,
	[NB_T_EREADER]			= T_CLASS_TAB,
	[NB_T_MEDIAPLAYER]		= T_CLASS_MOB,	// ipod, ms zune, yp-gb1
	[NB_T_MOBILECONSOLE]		= T_CLASS_MOB,
	[NB_T_MOBILE_BROWSER]		= T_CLASS_MOB,
	[NB_T_MOBILE_PHONE]		= T_CLASS_MOB,
	[NB_T_OPERATINGSYSTEM]		= T_CLASS_MOB,	// ios 4.2
	[NB_T_SETTOP_BOX_TV]		= T_CLASS_DSK,
	[NB_T_TABLET]			= T_CLASS_TAB,
	[NB_T_WEARABLE_COMPUTER] 	= T_CLASS_MOB	// android 4.0*glass
};

const char *
dcs_type_class(int type_id /* enum dcs_type */) {
	check_type_id(type_id, 0);
	return (dcs_type_class_str[dcs_type2class[type_id]]);
}
