/*
 * Copyright (c) 2014 UPLEX - Nils Goroll Systemoptimierung
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS _AS IS_ AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * common code for varnish 2 (inline-C) and varnish 3 (vmod)
 *
 * this gets _INCLUDED_
 */

#include "dcs_classifier.c"

#include <ctype.h>

#include "dcs_match.c"
#include "dcs_type.c"

#define UA_LIMIT 1024

#define DCS_VARNISH2_NHDRS 4
const char * const hdrs[DCS_VARNISH2_NHDRS] = {
	[0] = "\025X-OperaMini-Phone-UA:",
	[1] = "\024X-Device-User-Agent:",
	[2] = "\026X-Original-User-Agent:",
	[3] = "\016X-Goog-Source:"
};

const char * const ua_prepend[DCS_VARNISH2_NHDRS] = {
	[0] = " opera/ opera mini/  ",
	[1] = NULL,
	[2] = NULL,
	[3] = NULL
};

#define appnd(w, space, r, l)			\
	l = strlen(r);				\
	strncpy(w, r, space);			\
	if (l > space) {			\
		w += space;			\
		space = 0;			\
		break;				\
	}					\
	space -= l;				\
	w += l;

static int
dcs_varnish_classify(const struct sess *sp) {
	const char *ua = VRT_GetHdr(sp, HDR_REQ, "\013User-Agent:");
	const char *r;
	char *w = NULL;

	int i;

	size_t l;
	size_t space = UA_LIMIT;
	char   uabuf[space];

	if ((! ua) || (! *ua))
		return NB_E_SPECIAL_UNIDENTIFIED;

	if (VRT_GetHdr(sp, HDR_REQ, "\016x-wap-profile:"))
		return NB_E_SPECIAL_GENERIC_WAP;

	/* we need to copy to downcase the string for matching */
	w = uabuf;
	do {
		appnd(w, space, ua, l);
	} while(0);
	ua = uabuf;
	
	for (i = 0; i < DCS_VARNISH2_NHDRS; i++) {
		r = VRT_GetHdr(sp, HDR_REQ, hdrs[i]);
		if (r && *r) {
			if (ua_prepend[i])
				appnd(w, space, ua_prepend[i], l);
			appnd(w, space, r, l);
		}
	}

	uabuf[UA_LIMIT] = '\0';

	assert(ua == uabuf);
	assert(w > uabuf);
	assert(w <= (uabuf + UA_LIMIT));
	
	for (w = uabuf; *w; ++w) *w = tolower(*w);

	return dcs_match(ua);
}
